/// <reference path="game.ts" />
var RogueRpg;
(function (RogueRpg) {
    "use strict";
    // The renderer should take all rendering code, such that game.ts is not worried
    // about rendering on the canvas.
    // A separate file will handle all browser UI handling
    var Renderer = /** @class */ (function () {
        function Renderer(game) {
            this.game = game;
            this.useSimpleGraphics = false;
            this.useOneCanvas = false;
            this.showDamageText = true;
            this.animateLevelUpTimer = 0;
            this.renderGemstones = true;
            this.renderFloorSection = true;
            this.renderShardsSection = true;
            this.renderMinimap = true;
            this.renderLevelBar = true;
            this.renderItemCount = true;
            this.levelUpTextWidth = 0;
            this.levelUpAnimTimer = 0;
            // Draws a rotating starburst centered on the position. Handles its own rotation (LIMIT 1 right now)
            this.starburstRotation = 0;
            this.dpr = window.devicePixelRatio;
            this.antialias = false;
            this.useSimpleGraphics = false;
            this.useOneCanvas = this.useSimpleGraphics || RogueRpg.Game.testCrashFix;
            this.canvas = by.id("canvas");
            this.setupHdpiCanvas(this.canvas);
            this.context = this.canvas.getContext("2d");
            this.staticCanvas = by.id("staticCanvas");
            this.setupHdpiCanvas(this.staticCanvas, true);
            this.staticContext = this.staticCanvas.getContext("2d");
            if (!game.tutorialCompleted) {
                this.tutorialCanvas = this.setupStandardCanvas("tutorialCanvas");
                this.tutorialContext = this.tutorialCanvas.getContext("2d");
            }
            this.setupStandardCanvas("screenshotCanvas");
            // Hide the inventory button
            this.setUseOneCanvas(this.useSimpleGraphics || RogueRpg.Game.testCrashFix);
        }
        Renderer.prototype.setUseOneCanvas = function (newVal) {
            if (newVal) {
                by.id('inventoryAccess').style.display = "none";
                this.staticCanvas.style.display = "none";
                this.staticContext = this.context;
            }
            else {
                by.id('inventoryAccess').style.display = "block";
                this.staticCanvas.style.display = "block";
                this.staticContext = this.staticCanvas.getContext("2d");
            }
        };
        // Sets up a canvas at the device's pixel ratio, no antialiasing
        Renderer.prototype.setupStandardCanvas = function (id) {
            var scale = this.dpr;
            if (scale > 1) {
                scale = 1;
            }
            var canvas = by.id(id);
            var context = canvas.getContext("2d");
            canvas.width = this.ww * scale;
            canvas.height = this.wh * scale;
            canvas.style.width = this.ww + "px";
            canvas.style.height = this.wh + "px";
            context.textAlign = "left";
            context.textBaseline = "top";
            context.imageSmoothingEnabled = this.antialias;
            return canvas;
        };
        // Given a canvas that is on the page with proper scaling, etc and full-screen
        // Currently, that canvas needs its own CSS
        // TODO: If I want to use a lowDpi canvas, maybe there is a better way?
        // So if I toggle to a canvas that is 1:1 or less, I don't use this algorithm, such as 'scale'?
        Renderer.prototype.setupHdpiCanvas = function (canvas, isStatic) {
            if (isStatic === void 0) { isStatic = false; }
            var useAntialiasing = isStatic ? false : this.antialias;
            var scale = isStatic ? Math.min(window.devicePixelRatio, 2) : this.dpr;
            if (this.useSimpleGraphics || scale <= 0) {
                this.setupStandardCanvas(canvas.id);
                return;
            }
            var context = canvas.getContext("2d");
            this.ww = window.innerWidth;
            this.wh = window.innerHeight;
            canvas.width = this.ww * scale;
            canvas.height = this.wh * scale;
            canvas.style.width = this.ww + "px";
            canvas.style.height = this.wh + "px";
            context.textAlign = "left";
            context.textBaseline = "top";
            context.scale(scale, scale);
            context.imageSmoothingEnabled = useAntialiasing;
        };
        // Note that these should be unused right now
        Renderer.prototype.generateCanvases = function (draw) {
            this.game.average = 0;
            this.game.deltaTimes = [];
            this.setupHdpiCanvas(this.canvas);
            this.setupHdpiCanvas(this.staticCanvas, true);
            this.setupStandardCanvas("screenshotCanvas");
            if (!this.game.tutorialCompleted) {
                //this.refreshExistingCanvas(this.tutorialCanvas, true);
            }
            this.staticContext.clearRect(0, 0, this.ww, this.wh);
            this.resetStaticContext();
            if (draw) {
                this.render(RogueRpg.Helpers.game);
            }
        };
        Renderer.prototype.changeDpr = function () {
            // TODO: Canvas was flashing white on my S6 when I allowed 0.5
            if (this.dpr == 1 / 4) {
                this.dpr = 4;
            }
            else if (this.dpr <= 1) {
                this.dpr = window.devicePixelRatio; // this.dpr / 2; // window.devicePixelRatio;  //
            }
            else {
                // Just in case it ever gets off a whole number
                this.dpr = Math.round(this.dpr - 1);
            }
            this.generateCanvases(true);
        };
        Renderer.prototype.changeAA = function () {
            this.antialias = !this.antialias;
            this.canvas.style.imageRendering = this.antialias ? "auto" : "pixelated";
            this.generateCanvases(true);
        };
        Renderer.prototype.setUseSimpleGraphics = function (newVal) {
            this.useSimpleGraphics = newVal;
            this.useOneCanvas = newVal;
            this.setUseOneCanvas(newVal);
            this.generateCanvases(true);
        };
        Renderer.prototype.resetStaticContext = function () {
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(0, 0, this.ww, this.wh);
            }
            this.renderMinimap = true;
            this.renderGemstones = true;
            this.renderFloorSection = true;
            this.renderLevelBar = true;
            this.renderShardsSection = true;
            this.renderItemCount = true;
        };
        Renderer.prototype.render = function (game) {
            if (this.wh !== window.innerHeight || this.ww != window.innerWidth) {
                this.wh = window.innerHeight;
                this.ww = window.innerWidth;
                this.generateCanvases();
            }
            this.originX = (RogueRpg.Helpers.OriginX);
            this.originY = (RogueRpg.Helpers.OriginY);
            if (this.useSimpleGraphics) {
                //this.originX = Math.round(this.originX);
                //this.originY = Math.round(this.originY);
            }
            if (this.useOneCanvas) {
                this.resetStaticContext();
            }
            this.context.fillStyle = "rgb(0, 0, 0)";
            this.context.fillRect(0, 0, this.ww, this.wh);
            this.drawWorldTiles(game.map); // BAD?
            this.drawGemstonesAndShards(game.map);
            this._drawHealingItems(game.map);
            this._updateAndDrawFloatingText(game.floatingText);
            this.drawItems();
            this.drawParticles();
            this.drawUnits();
            this.drawProjectiles();
            this.drawAnimations();
            //this._drawDebugPoints();
            this.drawMessages();
            this.drawHeroLifeBar(game.hero);
            this.updateAndDrawLostChunks(game.losingLifeChunks);
            this.animateLevelUp(game.hero);
            if (game.recordTiming) {
                this.context.fillStyle = "white";
                this.context.fillText("frame: " + Math.round(game.average * 10) / 10 + "ms", 20, 150);
                this.context.fillText("loop:  " + Math.round(game.loopAverage * 10) / 10 + "ms", 20, 175);
            }
            // TODO: This was used in the single canvas experiment
            //this.resetStaticContext();
            //this.staticContext = this.context;
            //this.staticCanvas.style.display = "none";
            this.drawStaticUi();
        };
        Renderer.prototype.drawMessages = function () {
            if (this.game.flyingKickCount > 4) {
                this.context.fillStyle = "white";
                this.context.font = "24pt Arial";
                // TODO: Center
                var text = "Combo: " + this.game.flyingKickCount + "!";
                var width = this.context.measureText(text).width;
                this.context.fillText(text, this.ww / 2 - width / 2, 150);
            }
        };
        Renderer.prototype.drawFloorTiles = function (startI, startJ, endI, endJ) {
            var map = this.game.map;
            // TODO: Maybe draw decor fully after, so it can extend over floor?
            for (var i = startI; i < endI; i++) {
                for (var j = startJ; j < endJ; j++) {
                    map.tiles[i][j].seen = true;
                    if (map.tiles[i][j].type) {
                        var type = map.tiles[i][j].type;
                        if (type.indexOf("Wall") == -1) {
                            var offsetX = (j * RogueRpg.Helpers.TileSize) - this.originX;
                            var offsetY = (i * RogueRpg.Helpers.TileSize) - this.originY;
                            this.context.drawImage(RogueRpg.Images.get(type), offsetX, offsetY, RogueRpg.Helpers.TileSize, RogueRpg.Helpers.TileSize);
                        }
                    }
                }
            }
        };
        Renderer.prototype.drawWallTiles = function (startI, startJ, endI, endJ) {
            var map = this.game.map;
            for (var i = startI; i < endI; i++) {
                for (var j = startJ; j < endJ; j++) {
                    map.tiles[i][j].seen = true;
                    if (map.tiles[i][j].type) {
                        var type = map.tiles[i][j].type;
                        if (type.indexOf("Wall") > 0) {
                            var offsetX = (j * RogueRpg.Helpers.TileSize) - this.originX;
                            var offsetY = (i * RogueRpg.Helpers.TileSize) - this.originY;
                            this.context.drawImage(RogueRpg.Images.get(type), offsetX, offsetY, RogueRpg.Helpers.TileSize, RogueRpg.Helpers.TileSize);
                            if (map.tiles[i][j].decor) {
                                this.context.drawImage(RogueRpg.Images.get(map.tiles[i][j].decor), offsetX, offsetY, RogueRpg.Helpers.TileSize, RogueRpg.Helpers.TileSize);
                            }
                        }
                    }
                }
            }
        };
        // TODO: Profile this and make sure it is okay
        // TODO: Add illumination sometime
        Renderer.prototype.illuminateFloor = function () {
            this.context.fillStyle = "rgba(0, 0, 0, 0.5)";
            this.context.fillRect(0, 0, this.ww, this.wh);
            this.context.globalCompositeOperation = "lighter";
            var x = RogueRpg.Helpers.HeroDrawX + 20;
            var y = RogueRpg.Helpers.HeroDrawY + 40;
            var r = 200;
            var grd = this.context.createRadialGradient(x, y, 30, x, y, r);
            grd.addColorStop(0, "rgba(255, 230, 170, 0.25)");
            grd.addColorStop(1, "rgba(255, 230, 170, 0)");
            this.context.fillStyle = grd;
            this.context.fillRect(x - r, y - r, x + r, y + r);
            this.context.globalCompositeOperation = "source-over";
        };
        Renderer.prototype.drawWorldTiles = function (map) {
            var startI = Math.max(0, Math.floor(this.originY / RogueRpg.Helpers.TileSize) - 1);
            var startJ = Math.max(0, Math.floor(this.originX / RogueRpg.Helpers.TileSize) - 1);
            var endI = Math.min(map.mapHeight, Math.ceil((this.originY + this.wh) / RogueRpg.Helpers.TileSize) + 1);
            var endJ = Math.min(map.mapWidth, Math.ceil((this.originX + this.ww) / RogueRpg.Helpers.TileSize) + 1);
            this.drawFloorTiles(startI, startJ, endI, endJ);
            //this.illuminateFloor();
            this.drawWallTiles(startI, startJ, endI, endJ);
        };
        Renderer.prototype._drawHealingItems = function (map) {
            for (var i = 0; i < map.healingOnGround.length; i++) {
                var item = map.healingOnGround[i];
                this.context.drawImage(RogueRpg.Images.get(item.name), item.drawX - this.originX, item.drawY - this.originY, 30, 30);
            }
        };
        Renderer.prototype.drawGemstonesAndShards = function (map) {
            for (var i = 0; i < map.gemstonesOnGround.length; i++) {
                var item = map.gemstonesOnGround[i];
                this.context.drawImage(RogueRpg.Images.get(item.name), item.drawX - this.originX, item.drawY - this.originY);
            }
            for (var i = 0; i < map.shardsOnGround.length; i++) {
                var shard = map.shardsOnGround[i];
                this.context.drawImage(RogueRpg.Images.get("shards"), shard.x - this.originX - 10, shard.y - this.originY - 10, 20, 20);
            }
            for (var i = 0; i < map.goldOnGround.length; i++) {
                var gold = map.goldOnGround[i];
                this.context.drawImage(RogueRpg.Images.get("Gold"), gold.x - this.originX - 10, gold.y - this.originY - 10, 20, 20);
            }
            for (var i = 0; i < map.potionsOnGround.length; i++) {
                var potion = map.potionsOnGround[i];
                this.context.drawImage(RogueRpg.Images.get(potion.potion.image), potion.drawLocation.x - this.originX - 15, potion.drawLocation.y - this.originY - 15, 30, 30);
            }
        };
        Renderer.prototype._updateAndDrawFloatingText = function (floatingText) {
            // Limit floating texts to 30, replace older text with newer, to avoid uber-lag
            if (floatingText.length > 30) {
                floatingText.splice(0, floatingText.length - 30);
            }
            this.context.lineWidth = 3;
            this.context.font = "24px Helvetica";
            for (var i = floatingText.length - 1; i >= 0; i--) {
                var text = floatingText[i];
                if (!this.showDamageText && text.combat) {
                    floatingText.splice(i, 1);
                    continue;
                }
                text.timeRemaining -= RogueRpg.Game.deltaTime;
                text.drawY -= text.speed;
                this.context.fillStyle = this.useSimpleGraphics ? RogueRpg.FloatingText.GetSimpleColor(text) : RogueRpg.FloatingText.GetColor(text);
                // TODO: Temporary experiment with integer location for text
                var x = Math.floor(text.drawX - this.originX);
                var y = Math.floor(text.drawY - this.originY);
                if (!this.useSimpleGraphics) {
                    this.context.strokeStyle = "rgba(0, 0, 0, " + RogueRpg.FloatingText.GetOpacity(text) + ")";
                    this.context.strokeText(text.text, x, y);
                }
                this.context.fillText(text.text, x, y);
                if (text.timeRemaining <= 0) {
                    floatingText.splice(i, 1);
                }
            }
        };
        Renderer.prototype.drawItems = function () {
            this.context.font = "24px Helvetica";
            for (var i = 0; i < this.game.map.itemsOnGround.length; i++) {
                var item = this.game.map.itemsOnGround[i];
                if (!item.drawHeight || !item.drawWidth) {
                    item.drawWidth = this.context.measureText(item.item.fullName).width;
                    item.drawHeight = 24;
                }
                var offsetX = item.drawX - this.originX;
                var offsetY = item.drawY - this.originY;
                if (offsetX < -item.drawWidth - 4 || offsetY < -item.drawHeight - 4 || offsetX > this.ww + 2 || offsetY > this.wh + 2) {
                    continue; // Do not try to render anything outside the canvas
                }
                this.context.fillStyle = "rgba(0, 0, 0, 0.7)";
                this.context.fillRect(offsetX - 2, offsetY - 2, item.drawWidth + 4, 30);
                this.context.fillStyle = RogueRpg.Helpers.RarityColors[item.item.rarity];
                this.context.fillText(item.item.fullName, offsetX, offsetY);
            }
        };
        Renderer.prototype.drawParticles = function () {
            RogueRpg.ParticleEmitter.UpdateAndRenderAll(this.context, RogueRpg.Game.deltaTime);
        };
        Renderer.prototype.drawProjectiles = function () {
            for (var i = 0; i < this.game.projectiles.length; i++) {
                var p = this.game.projectiles[i];
                this.drawImageRotated(RogueRpg.Images.get(p.imageName), p.direction, p.x - this.originX, p.y - this.originY - p.zOffset, 30, 11);
            }
        };
        Renderer.prototype.drawUnits = function () {
            var hero = this.game.hero;
            var units = [hero].concat(this.game.map.monsters, this.game.map.deadMonsters);
            if (hero.state.pets) {
                units = units.concat(hero.state.pets);
            }
            // units.concat(this._map.specialWorldObjects);
            for (var i = 0; i < this.game.map.specialWorldObjects.length; i++) {
                var obj = this.game.map.specialWorldObjects[i];
                if (!obj.x) {
                    obj.x = obj.drawX + obj.drawWidth / 2;
                }
                if (!obj.y) {
                    obj.y = obj.drawY + obj.drawHeight;
                }
                units.push(obj);
            }
            if (RogueRpg.Level[RogueRpg.L.Orb]) {
                for (var i = 0; i < hero.orbPositions.length; i++) {
                    units.push({ x: hero.orbPositions[i].x, y: hero.orbPositions[i].y, orb: true });
                }
            }
            units.sort(function (a, b) {
                return a.y - b.y;
            });
            for (var i = 0; i < units.length; i++) {
                var u = units[i];
                if (u == hero) {
                    this._drawHero(u);
                }
                else if (this.game.map.deadMonsters.indexOf(u) >= 0) {
                    this.drawDeadMonster(u, this.game.map);
                }
                else if (this.game.map.specialWorldObjects.indexOf(u) >= 0) {
                    this.drawSpecialWorldObject(u);
                }
                else if (u['orb']) {
                    var x = u.x - this.originX - 15; // TODO: Magic number size
                    var y = u.y - this.originY - 15;
                    this.context.drawImage(RogueRpg.Images.get("Orb"), x, y - 20, 30, 30);
                }
                else {
                    this.drawDude(u);
                }
            }
        };
        Renderer.prototype.drawSpecialWorldObject = function (obj) {
            var offsetX = obj.drawX - this.originX;
            var offsetY = obj.drawY - this.originY;
            if (offsetX < -obj.drawWidth || offsetY < -obj.drawHeight || offsetX > this.ww || offsetY > this.wh) {
                return; // Do not try to render anything outside the canvas
            }
            if (obj.type == "TestMonster") {
                var mon = obj;
                RogueRpg.TestMonsterWorldObject.Update(mon);
                this.drawDude(mon.monster);
            }
            else if (obj.type.indexOf(" World") > 0 || obj.type.indexOf("Scumbasher") >= 0 || obj.type === "Potion Slot") {
                // This item is a relic and should be treated specially
                // First, draw pedestal
                // Next, increment motion by sine wave
                // Finally, draw item itself
                this.context.drawImage(RogueRpg.Images.get("Pedestal"), offsetX, offsetY);
                obj.data = obj.data || { t: 0 };
                obj.data.t += 1.5 * RogueRpg.Game.deltaTime / 1000;
                this.context.drawImage(RogueRpg.Images.get(obj.type), offsetX - 3, offsetY - 22 + Math.sin(obj.data.t) * 8, 50, 50);
            }
            else {
                this.context.drawImage(RogueRpg.Images.get(obj.type), offsetX, offsetY, obj.drawWidth, obj.drawHeight);
            }
        };
        Renderer.prototype._drawHero = function (hero) {
            if (this.game.clearingCurrentLevel) {
                var img = RogueRpg.Images.get("hero fly1");
                this.drawImageRotated(img, this.game.clearingCurrentLevelAngle + Math.PI / 2, RogueRpg.Helpers.HeroDrawX, RogueRpg.Helpers.HeroDrawY, RogueRpg.Helpers.HeroWidth, RogueRpg.Helpers.HeroHeight, RogueRpg.Helpers.HeroWidth / 2, RogueRpg.Helpers.HeroHeight / 2);
                return;
            }
            var weaponSize = 40;
            var shieldSize = 35;
            var face = RogueRpg.Mood.GetHeroFace();
            this.context.drawImage(face, RogueRpg.Helpers.HeroDrawX + 4, RogueRpg.Helpers.HeroDrawY + 9);
            if (hero.facingRight) {
                this.context.save();
                this.context.translate(hero.x - this.originX, 0);
                this.context.scale(-1, 1);
                this.context.translate(-hero.x + this.originX, 0);
            }
            var draw = RogueRpg.Mood.GetHeroImage();
            this.context.drawImage(RogueRpg.Images.get(draw.image), RogueRpg.Helpers.HeroDrawX, RogueRpg.Helpers.HeroDrawY, RogueRpg.Helpers.HeroWidth, RogueRpg.Helpers.HeroHeight);
            if (hero.equippedArmor) {
                this.context.drawImage(RogueRpg.Images.get(hero.equippedArmor.image), RogueRpg.Helpers.HeroDrawX, RogueRpg.Helpers.HeroDrawY + 29, 40, 40);
            }
            if (hero.equippedWeapon) {
                var wx = RogueRpg.Helpers.HeroDrawX + RogueRpg.HeroRender.defaultWeapon.x + draw.weaponAttachmentPoint.x - weaponSize;
                var wy = RogueRpg.Helpers.HeroDrawY + RogueRpg.HeroRender.defaultWeapon.y + draw.weaponAttachmentPoint.y - weaponSize;
                if (draw.weaponRotation == 0) {
                    this.context.drawImage(RogueRpg.Images.get(hero.equippedWeapon.image), wx, wy, weaponSize, weaponSize);
                }
                else {
                    this.drawImageRotated(RogueRpg.Images.get(hero.equippedWeapon.image), draw.weaponRotation * (Math.PI / 180), wx, wy, weaponSize, weaponSize, weaponSize, weaponSize);
                }
            }
            if (hero.equippedShield) {
                var sx = RogueRpg.Helpers.HeroDrawX + RogueRpg.HeroRender.defaultShield.x + draw.shieldAttachmentPoint.x - shieldSize / 2;
                var sy = RogueRpg.Helpers.HeroDrawY + RogueRpg.HeroRender.defaultShield.y + draw.shieldAttachmentPoint.y - shieldSize / 2;
                if (draw.shieldRotation == 0) {
                    this.context.drawImage(RogueRpg.Images.get(hero.equippedShield.image), sx, sy, shieldSize, shieldSize);
                }
                else {
                    this.drawImageRotated(RogueRpg.Images.get(hero.equippedShield.image), draw.shieldRotation * (Math.PI / 180), sx, sy, shieldSize, shieldSize, shieldSize / 2, shieldSize / 2);
                }
            }
            var heroClass = RogueRpg.HeroClass.Classes[hero.getHeroState().classIndex];
            if (heroClass.hat) {
                this.context.drawImage(RogueRpg.Images.get(heroClass.hat), RogueRpg.Helpers.HeroDrawX, RogueRpg.Helpers.HeroDrawY - 9, 40, 40);
            }
            if (hero.facingRight) {
                this.context.restore();
            }
            // this._drawLifeBar(this.hero, Helpers.HeroDrawX, Helpers.HeroDrawY - 20);
            this._drawSpecialTimerBar(hero, RogueRpg.Helpers.HeroDrawX, RogueRpg.Helpers.HeroDrawY - 13);
        };
        Renderer.prototype.drawDude = function (dude) {
            var offsetX = dude.drawX - this.originX + (dude.drawWidth - 40) / 2;
            var offsetY = dude.drawY - this.originY;
            var imgType = dude.type;
            if (dude.isPet) {
                var x = Math.random();
            }
            var drawWidth = dude.drawWidth;
            var drawHeight = dude.drawHeight;
            if (dude.attackingTimer > 0) {
                dude.attackingTimer -= RogueRpg.Game.deltaTime;
                if (!dude.attackImages) {
                    var angleRadians = Math.atan2(this.game.hero.y - dude.y, this.game.hero.x - dude.x);
                    var movementX = Math.cos(angleRadians) * 5;
                    var movementY = Math.sin(angleRadians) * 5;
                    offsetX += movementX;
                    offsetY += movementY;
                }
            }
            if (offsetX < -dude.drawWidth || offsetY < -dude.drawHeight || offsetX > this.ww || offsetY > this.wh) {
                return; // Do not try to render anything outside the csnvas
            }
            var image;
            // var origImage = Images.get(imgType);
            var imageName = RogueRpg.DudeSpriteUtils.GetImageNameForDude(dude);
            if (dude.tookDamageTimer > 0) {
                image = RogueRpg.Helpers.MorphImage(imageName, dude.drawWidth, dude.drawHeight);
                imageName += RogueRpg.Helpers.MORPH_SUFFIX;
            }
            else {
                image = RogueRpg.Images.get(imageName);
            }
            if (dude.facingRight) {
                var offset = (Renderer.offsetMap[dude.type] || 0);
                var center = dude.drawWidth / 2;
                if (offset) {
                    center += offset;
                }
                image = RogueRpg.Helpers.MirrorImage(imageName, center, dude.drawWidth, dude.drawHeight);
                var delta = image.width - drawWidth; // the number of px offset
                drawWidth = image.width;
                drawHeight = image.height;
                this.context.drawImage(image, offsetX - delta / 2 - offset, offsetY, drawWidth, drawHeight); // - offset * 2 - (delta) added
            }
            else {
                var offset = (Renderer.offsetMap[dude.type] || dude.drawWidth / 8);
                this.context.drawImage(image, offsetX - offset * 2, offsetY, drawWidth, drawHeight);
            }
            if (dude.life < dude.maxLife) {
                this._drawLifeBar(dude, offsetX, offsetY);
            }
        };
        Renderer.prototype.drawDeadMonster = function (dm, map) {
            var i = map.deadMonsters.indexOf(dm);
            if (!RogueRpg.DeadMonster.Update(dm)) {
                map.deadMonsters.splice(i, 1);
                i--;
            }
        };
        Renderer.prototype.drawAnimations = function () {
            var active = this.game.activeAnimations;
            var sprite = this.game.spriteAnimations;
            var bridged = this.game.bridgedAnimations;
            for (var i = active.length - 1; i >= 0; i--) {
                var a = active[i];
                var offsetX = a.drawX - this.originX;
                var offsetY = a.drawY - this.originY;
                var image = RogueRpg.Images.get(a.name);
                var imageWidth = image.width;
                var imageHeight = image.height;
                if (a.rotation) {
                    // TODO: Don't just manually scale here
                    var height = imageHeight * 2 / 3;
                    var width = imageWidth * 2 / 3;
                    this.drawImageRotated(image, a.rotation + Math.PI / 2, offsetX - width / 2, offsetY - height / 2, width, height, width / 2, height / 2);
                    if (a.velocity) {
                        a.drawX += Math.cos(a.rotation) * a.velocity * RogueRpg.Game.deltaTime / 1000;
                        a.drawY += Math.sin(a.rotation) * a.velocity * RogueRpg.Game.deltaTime / 1000;
                    }
                    if (a.drag) {
                        a.velocity -= a.drag * RogueRpg.Game.deltaTime / 1000;
                        if (a.velocity < 0) {
                            a.velocity = 0;
                        }
                    }
                }
                else {
                    this.context.drawImage(image, offsetX - imageWidth / 2, offsetY - imageHeight / 2);
                }
                a.timeElapsed += RogueRpg.Game.deltaTime;
                if (a.timeElapsed > a.maxTime) {
                    active.splice(i, 1);
                }
            }
            for (var i = sprite.length - 1; i >= 0; i--) {
                var anim = sprite[i];
                if (anim.update(RogueRpg.Game.deltaTime)) {
                    anim.draw(this.context);
                }
                else {
                    sprite.splice(i, 1);
                }
            }
            for (var i = bridged.length - 1; i >= 0; i--) {
                var b = bridged[i];
                this.drawImageBridging({ x: b.x1, y: b.y1 }, { x: b.x2, y: b.y2 }, b.image);
                b.timeLeft -= RogueRpg.Game.deltaTime;
                if (b.timeLeft <= 0) {
                    bridged.splice(i, 1);
                }
            }
        };
        // Loads an image, such as a lightning bolt, then draws it rotated and scaled so its horizontal center line bridges two points
        Renderer.prototype.drawImageBridging = function (start, end, imageString) {
            var image = RogueRpg.Images.get(imageString);
            var dist = RogueRpg.Helpers.dist({ x: start.x, y: start.y }, { x: end.x, y: end.y });
            var scaling = dist / image.width;
            // this.context.drawImage(image, this.originX - this.hero.x, this.originY - this.hero.y - image.height / 2, image.width * scaling, image.height * scaling);
            this.drawImageRotated(image, Math.atan2(end.y - start.y, end.x - start.x), start.x - this.originX, start.y - this.originY - (image.height / 2) * scaling, image.width * scaling, image.height * scaling);
        };
        Renderer.prototype._drawDebugPoints = function () {
            this.context.fillStyle = "rgb(255, 255, 255)";
            this.context.fillRect(this.game.hero.x - this.originX, this.game.hero.y - this.originY, 4, 4);
            this.context.fillStyle = "rgb(255, 0, 255)";
            for (var i = 0; i < this.game.map.monsters.length; i++) {
                var m = this.game.map.monsters[i];
                this.context.fillRect(m.x - this.originX - 2, m.y - this.originY - 2, 4, 4);
            }
            this.context.fillStyle = "rgb(255, 125, 255)";
            for (var i = 0; i < this.game.hero.state.pets.length; i++) {
                var p = this.game.hero.state.pets[i];
                this.context.fillRect(p.x - this.originX - 2, p.y - this.originY - 2, 4, 4);
            }
        };
        Renderer.prototype._drawSpecialTimerBar = function (hero, offsetX, offsetY) {
            var border = 1;
            var timerBarHeight = 7;
            var timerBarWidth = 45;
            // this.context.fillStyle = "rgb(255, 255, 255)";
            // this.context.fillRect(offsetX, offsetY - timerBarHeight, timerBarWidth, timerBarHeight);
            var timerFullPercent = (hero.specialDelayStart - Math.max(0, hero.specialDelay)) / hero.specialDelayStart;
            var renderSpace = (timerBarWidth - border * 2);
            if (timerFullPercent > 0.999) {
                this.context.fillStyle = "rgb(255, 255, 0)";
            }
            else {
                this.context.fillStyle = "rgb(200, 200, 0)";
            }
            this.context.fillRect(offsetX + border, offsetY - timerBarHeight + border, timerFullPercent * renderSpace, timerBarHeight - border * 2);
            //this.context.fillStyle = "rgb(255, 0, 0)";
            //this.context.fillRect(offsetX + border + monsterLifePercent * renderSpace, offsetY - timerBarHeight + border,(1 - monsterLifePercent) * renderSpace, timerBarHeight - border * 2);
        };
        Renderer.prototype._drawLifeBar = function (unit, offsetX, offsetY) {
            var border = 1;
            var lifeBarHeight = 13;
            var lifeBarWidth = 45;
            this.context.fillStyle = "rgb(245, 245, 245)";
            this.context.fillRect(offsetX, offsetY - lifeBarHeight, lifeBarWidth, lifeBarHeight);
            var monsterLifePercent = Math.max(0, Math.min(1, unit.life / unit.maxLife));
            var renderSpace = (lifeBarWidth - border * 2);
            this.context.fillStyle = "rgb(0, 215, 0)";
            this.context.fillRect(offsetX + border, offsetY - lifeBarHeight + border, monsterLifePercent * renderSpace, lifeBarHeight - border * 2);
            this.context.fillStyle = "rgb(235, 0, 0)";
            this.context.fillRect(offsetX + border + monsterLifePercent * renderSpace, offsetY - lifeBarHeight + border, (1 - monsterLifePercent) * renderSpace, lifeBarHeight - border * 2);
        };
        // TODO: Refactor 'border' out of 4 params
        Renderer.prototype.drawHeroLifeBar = function (hero) {
            var offsetY = 2;
            var offsetX = 36;
            var border = 1;
            var lifeBarHeight = 13;
            var lifeBarWidth = window.innerWidth - offsetX - 4;
            // TODO: Render in static context, clear. Note that it normally renders in normal
            this.context.fillStyle = "rgb(245, 245, 245)";
            this.context.fillRect(offsetX, offsetY, lifeBarWidth, lifeBarHeight);
            var lifePct = Math.max(0, Math.min(1, hero.life / hero.maxLife));
            var renderSpace = (lifeBarWidth - border * 2);
            this.context.fillStyle = "rgb(0, 145, 0)";
            this.context.fillRect(offsetX + border, offsetY + border, lifePct * renderSpace, lifeBarHeight - border * 2);
            this.context.fillStyle = "rgb(185, 0, 0)";
            this.context.fillRect(offsetX + border + lifePct * renderSpace, offsetY + border, (1 - lifePct) * renderSpace, lifeBarHeight - border * 2);
            this.context.font = "12px Lucida";
            this.context.fillStyle = "rgb(255, 255, 255)";
            this.context.fillText("生命:", 5, offsetY);
            // this.staticContext.fillText(Math.ceil(this.hero.life) + " / " + Math.ceil(this.hero.maxLife), Math.floor(lifeBarWidth / 2), offsetY);
            if (RogueRpg.Level[RogueRpg.L.Bloodied]) {
                this.context.fillStyle = "rgb(60, 60, 60)";
                this.context.fillRect(offsetX + border + 0.499 * renderSpace, offsetY + border, 0.501 * renderSpace, lifeBarHeight - border * 2);
            }
        };
        // For parameters to this function do not include border information
        Renderer.prototype.updateAndDrawLostChunks = function (chunks) {
            var bar = { x: 37, y: 3, width: this.ww - 40, height: 11 };
            for (var i = 0; i < chunks.length; i++) {
                var chunk = chunks[i];
                // draw the chunk
                var x = bar.x + bar.width * chunk.offsetAsPercent;
                var percentDone = (RogueRpg.LosingLifeChunk.DURATION - chunk.timeRemaining) / RogueRpg.LosingLifeChunk.DURATION;
                var y = bar.y + 80 * percentDone;
                var opacity = 1 - percentDone * percentDone;
                this.context.fillStyle = "rgba(255, 255, 255, " + opacity + ")";
                this.context.fillRect(x, y, bar.width * chunk.widthAsPercent, bar.height);
                // Now update it and see if it should go away
                var stillAlive = RogueRpg.LosingLifeChunk.Update(chunk);
                if (!stillAlive) {
                    chunks.splice(i, 1);
                    i--;
                }
            }
        };
        Renderer.prototype.animateLevelUp = function (hero) {
            if (this.animateLevelUpTimer > 3500) {
                this.context.fillStyle = "rgba(255, 255, 255, " + (4000 - this.animateLevelUpTimer) / 600 + ")";
                this.context.fillRect(0, 0, this.ww, this.wh);
            }
            else if (this.animateLevelUpTimer > 3000) {
                this.context.fillStyle = "rgba(255, 255, 255, " + (this.animateLevelUpTimer - 3200) / 600 + ")";
                this.context.fillRect(0, 0, this.ww, this.wh);
            }
            if (this.animateLevelUpTimer < 3600 && this.animateLevelUpTimer > 2000) {
                this.context.fillStyle = "rgb(255, 25, 25)";
                this.context.font = "bold 36px 'Lucida Sans'";
                var width = this.context.measureText("Level " + hero.level).width;
                this.context.strokeStyle = "rgb(255, 255, 255)";
                this.context.lineWidth = 8;
                this.context.strokeText("Level " + hero.level, this.ww / 2 - width / 2, 80);
                this.context.fillText("Level " + hero.level, this.ww / 2 - width / 2, 80);
            }
            this.animateLevelUpTimer -= RogueRpg.Helpers.LoopTime;
        };
        // Note that all of these only re-render when state has changed
        Renderer.prototype.drawStaticUi = function () {
            if (RogueRpg.Helpers.game.time() % 100 === 0) {
                this.resetStaticContext();
            }
            this.drawMiniMap(); // Comes before the others because it makes them rerender
            this.drawGemstonesSection();
            this.drawShardsSection();
            this.drawGoldSection();
            this.drawPrismaSection();
            this.drawFloorSection();
            this.drawItemCountBox();
            this.drawExpBar();
            this.drawHealingPotionButton();
        };
        Renderer.prototype.drawHealingPotionButton = function () {
            var size = Renderer.HealingPotionSize;
            var d = { left: this.ww - Renderer.HealingPotionRightOffset, top: this.wh - Renderer.HealingPotionBottomOffset };
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(d.left - 15, d.top - 15, size + 30, size + 30);
            }
            this.staticContext.drawImage(RogueRpg.Images.get("Potion Slot"), d.left, d.top, size, size);
            if (this.game.healingPotionButtonDown) {
                this.drawStarburstAt(d.left + size / 2, d.top + size / 2, 50);
            }
            // Check to see if any healing potions remain
            if (RogueRpg.Helpers.game.hero.state.healingPotionCount > 0) {
                this.staticContext.drawImage(RogueRpg.Images.get("PotionRed"), d.left + 3, d.top + 3, size - 6, size - 6);
            }
            else {
                // Nothing to draw here!
            }
            // Draw the potion count
            this.staticContext.fillStyle = "black";
            this.staticContext.strokeStyle = "white";
            this.staticContext.lineWidth = 2;
            this.staticContext.font = "15px 'Lucida Sans'";
            var potionCount = this.game.hero.state.healingPotionCount;
            var textWidth = this.staticContext.measureText("" + potionCount).width;
            var textHeight = 20;
            var width = Math.max(20, textWidth + 8);
            var left = d.left - width / 2 + 5;
            var top = d.top - textHeight / 2 + 3;
            this.staticContext.strokeRect(left, top, width, textHeight);
            this.staticContext.fillRect(left, top, width, textHeight);
            this.staticContext.fillStyle = "white";
            this.staticContext.fillText("" + potionCount, left + 6, top + 2);
        };
        // Starting at 20, 70 right up from bottom left, draw a pixel for each map tile that has been seen
        Renderer.prototype.drawMiniMap = function () {
            if (!this.renderMinimap) {
                return;
            }
            this.renderMinimap = false;
            this.renderGemstones = true;
            this.renderShardsSection = true;
            var map = this.game.map;
            var scale = 3;
            var startY = this.wh - 90 - map.mapHeight * scale;
            var height = map.mapHeight * scale;
            var startX = 10;
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(0, this.wh - 390, 60 * scale + startX + 50, 300);
            }
            var startI = Math.max(0, map.topMostRoom - 3);
            var startJ = Math.max(0, map.leftMostRoom - 3);
            var endI = Math.min(map.mapHeight, map.bottomMostRoom + 3);
            var endJ = Math.min(map.mapWidth, map.rightMostRoom + 3);
            for (var i = startI; i < endI; i++) {
                for (var j = startJ; j < endJ; j++) {
                    if (map.tiles[i] && map.tiles[i][j] && (map.tiles[i][j].seen || RogueRpg.Game.alwaysShowMiniMap)) {
                        if (map.tiles[i][j].walkable) {
                            this.staticContext.fillStyle = "rgb(155, 155, 155)";
                            this.staticContext.fillRect(startX + j * scale, startY + i * scale, scale + 0.2, scale + 0.2);
                        }
                        else {
                            // Draw the 'explored' color
                            this.staticContext.fillStyle = "rgba(180, 180, 180, 0.18)";
                            this.staticContext.fillRect(Math.floor(startX + j * scale), Math.floor(startY + i * scale), scale, scale);
                        }
                    }
                }
            }
            var heroLocation = RogueRpg.Helpers.ToTileLocation(this.game.hero.x, this.game.hero.y);
            this.staticContext.fillStyle = "rgb(255, 255, 255)";
            this.staticContext.fillRect(startX + heroLocation.x * scale, startY + heroLocation.y * scale, scale, scale);
            if (RogueRpg.Game.renderPotionsOnMiniMap) {
                for (var i = 0; i < map.potionsOnGround.length; i++) {
                    var dp = map.potionsOnGround[i];
                    var position = RogueRpg.Helpers.ToTileLocation(dp.drawLocation.x, dp.drawLocation.y);
                    this.staticContext.fillStyle = "rgba(255, 80, 255, 1)";
                    this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                }
            }
            var exitX = -1, exitY = -1;
            for (var i = 0; i < map.specialWorldObjects.length; i++) {
                var obj = map.specialWorldObjects[i];
                var position = RogueRpg.Helpers.ToTileLocation(obj.drawX + obj.drawWidth / 2, obj.drawY + obj.drawHeight / 2);
                if (map.tiles[position.y][position.x].seen || RogueRpg.Game.alwaysShowMiniMap) {
                    if (obj.type == "Level Exit") {
                        exitX = position.x;
                        exitY = position.y;
                    }
                    else if (obj.type == "Item Upgrade Shrine") {
                        this.staticContext.fillStyle = "rgba(255, 255, 0, 1)";
                        this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                    }
                    else if (obj.type.indexOf("Treasure Chest") >= 0 && obj.type.indexOf("Used") === -1) {
                        this.staticContext.fillStyle = "rgba(200, 200, 100, 1)";
                        this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                    }
                    else if (obj.type == "Healing Shrine") {
                        this.staticContext.fillStyle = "rgba(255, 100, 100, 1)";
                        this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                    }
                    else if (["Sitting Wizard", "Portal", "Ghost Girl"].indexOf(obj.type) >= 0) {
                        this.staticContext.fillStyle = "rgba(100, 150, 200, 1)";
                        this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                    }
                    else if (obj.type.indexOf(" World") > 0 || obj.type.indexOf("Scumbasher") >= 0 || obj.type === "Potion Slot") {
                        this.staticContext.fillStyle = "rgba(255, 100, 255, 1)";
                        this.staticContext.fillRect(startX + position.x * scale, startY + position.y * scale, scale, scale);
                    }
                }
            }
            if (exitX > 0 && exitY > 0) {
                this.staticContext.fillStyle = "rgba(100, 250, 250, 1)";
                this.staticContext.fillRect(startX + exitX * scale, startY + exitY * scale, scale, scale);
            }
        };
        Renderer.prototype.drawSection = function (section, imgName, count, useRed) {
            var d = section;
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(d.left - 10, this.wh - d.bottom - d.height - 10, d.height + 20, d.width + 20);
            }
            var border = 1;
            this.staticContext.fillStyle = "white";
            this.staticContext.fillRect(d.left, this.wh - d.bottom, d.width, d.height);
            this.staticContext.fillStyle = useRed ? "red" : "black";
            this.staticContext.fillRect(d.left + border, this.wh - (d.bottom - border), d.width - border * 2, d.height - border * 2);
            this.staticContext.fillStyle = "white";
            this.staticContext.font = "12px consolas";
            var gText = RogueRpg.Helpers.formatNumber(count);
            var w = this.staticContext.measureText(gText).width;
            this.staticContext.fillText(gText, d.left + d.width - 5 - w, this.wh - d.bottom + 4);
            var img = RogueRpg.Images.get(imgName);
            if (img != RogueRpg.Images.placeholder) {
                this.renderGemstones = false;
            }
            this.staticContext.drawImage(img, d.left + 2, this.wh - d.bottom + 1, 18, 18);
            return img !== RogueRpg.Images.placeholder;
        };
        Renderer.prototype.drawGemstonesSection = function (withAddButton) {
            if (withAddButton === void 0) { withAddButton = true; }
            if (!this.renderGemstones) {
                return;
            }
            this.drawSection(Renderer.GemstoneSectionStart, "gemstones", this.game.gemstones, this.game.gemstoneSectionDown);
        };
        Renderer.prototype.drawShardsSection = function () {
            if (!this.renderShardsSection) {
                return;
            }
            var drawSuccess = this.drawSection(Renderer.ShardSectionStart, "shards", this.game.shards);
            if (drawSuccess) {
                this.renderShardsSection = false;
            }
        };
        Renderer.prototype.drawGoldSection = function () {
            this.drawSection(Renderer.GoldSectionStart, "Gold", this.game.prismaState.gold);
        };
        Renderer.prototype.drawPrismaSection = function () {
            this.drawSection(Renderer.PrismaSectionStart, "Prisma", this.game.prismaState.prisma);
        };
        Renderer.prototype.drawFloorSection = function () {
            if (!this.renderFloorSection) {
                return;
            }
            this.renderFloorSection = false;
            var d = Renderer.FloorSectionSpot;
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(d.left - 10, d.top, d.width + 50, d.height + 20);
            }
            this.staticContext.fillStyle = "rgba(40, 40, 40, 0.8)";
            this.staticContext.fillRect(d.left, d.top, d.width, d.height);
            this.staticContext.fillStyle = "white";
            this.staticContext.font = "12px 'Lucida Sans'";
            this.staticContext.fillText("楼层", d.left + 4, d.top + 7);
            var textW = this.staticContext.measureText("Floor:").width;
            this.staticContext.font = "20px 'Lucida Sans'";
            var floor = this.game.previousMapStack.length > 0 ? "??" : "" + this.game.floor;
            this.staticContext.fillText(floor, d.left + textW + 4, d.top + 2);
            var mult = this.game.hero.state.multiplier;
            if (mult > 1) {
                this.staticContext.fillText(mult + "x", d.left + textW + 40, d.top + 2);
            }
        };
        Renderer.prototype.drawItemCountBox = function () {
            if (!this.renderItemCount) {
                return;
            }
            this.renderItemCount = false;
            var d = Renderer.ItemCountPosition;
            if (this.useOneCanvas) {
                if (this.game.inventoryAccessDown) {
                    this.staticContext.drawImage(RogueRpg.Images.get("inventoryButton"), this.ww - 56, this.wh - 96, 42, 42);
                }
                else {
                    this.staticContext.drawImage(RogueRpg.Images.get("inventoryButton"), this.ww - 60, this.wh - 100, 50, 50);
                }
            }
            else {
                this.staticContext.clearRect(this.ww - d.right - d.width - 40, this.wh - d.bottom - d.height - 10, 300, 40);
            }
            this.staticContext.fillStyle = "black";
            this.staticContext.strokeStyle = "white";
            this.staticContext.lineWidth = 2;
            this.staticContext.font = "15px 'Lucida Sans'";
            var itemCount = this.game.hero.items.length;
            var textWidth = this.staticContext.measureText("" + itemCount).width;
            var width = Math.max(d.width, textWidth + 8);
            this.staticContext.strokeRect(this.ww - d.right - width, this.wh - d.bottom - d.height, width, d.height);
            this.staticContext.fillRect(this.ww - d.right - width, this.wh - d.bottom - d.height, width, d.height);
            this.staticContext.fillStyle = "white";
            this.staticContext.fillText("" + itemCount, this.ww - d.right - textWidth - 4, this.wh - d.bottom - d.height + 2);
        };
        Renderer.prototype.drawExpBar = function () {
            // if (!this.renderLevelBar) {
            //    return;
            // }
            this.renderLevelBar = false;
            var y = window.innerHeight - 15;
            var border = 1;
            var height = 5;
            var width = this.ww - 10 - 30;
            if (!this.useOneCanvas) {
                this.staticContext.clearRect(0, y, this.ww, 13);
                this.staticContext.clearRect(this.ww - this.levelUpTextWidth - 100, 0, this.levelUpTextWidth + 14 + 200, 196);
            }
            this.staticContext.font = "12px Lucida";
            this.staticContext.fillStyle = "rgb(255, 255, 255)";
            this.staticContext.fillText("经验:", 5, y);
            this.staticContext.fillRect(35, y + 5, width, border); // Top border
            this.staticContext.fillRect(35, y + 5 + height + border, width, border); // bottom border
            this.staticContext.fillRect(35, y + 5, 1, height + border); // left edge
            this.staticContext.fillRect(35 + width, y + 5, 1, height + border); // right edge
            this.staticContext.fillStyle = "rgb(125, 255, 255)";
            this.staticContext.fillRect(36, y + 6, (width - 2) * (this.game.hero.currentExperience / this.game.hero.neededExperience), 2);
            this.staticContext.fillStyle = "rgb(95, 205, 205)";
            this.staticContext.fillRect(36, y + 6 + 2, (width - 2) * (this.game.hero.currentExperience / this.game.hero.neededExperience), 3);
            if (this.game.hero.state.levelUpPoints > 0) {
                this.levelUpAnimTimer = (this.levelUpAnimTimer + RogueRpg.Game.deltaTime) % 2000;
                var ratio = Math.abs(this.levelUpAnimTimer - 1000) / 2000;
                var gradColor = new RogueRpg.Rgba(255, 225, 25, ratio);
                RogueRpg.Utils.drawRadialGradient(this.staticContext, this.ww - 20 - (this.levelUpTextWidth / 2), 55, 80, gradColor);
                this.staticContext.font = "24px Helvetica";
                this.levelUpTextWidth = this.levelUpTextWidth || this.staticContext.measureText("Level Up!").width;
                this.staticContext.strokeStyle = "rgb(255, 225, 25)";
                this.staticContext.lineWidth = 2;
                this.staticContext.beginPath();
                this.staticContext.rect(this.ww - this.levelUpTextWidth - 20, 40, this.levelUpTextWidth + 10, 32);
                this.staticContext.stroke();
                this.staticContext.fillStyle = "rgb(255, 225, 25)";
                this.staticContext.fillText("升级技能!", this.ww - this.levelUpTextWidth - 15, 42);
                this.renderLevelBar = true;
            }
        };
        Renderer.prototype.drawImageMirrored = function (image, x, y, center, width, height) {
            this.context.save();
            this.context.translate(center - this.originX, 0);
            this.context.scale(-1, 1);
            this.context.translate(-(center - this.originX), 0);
            if (width && height) {
                this.context.drawImage(image, x, y, width, height);
            }
            else {
                this.context.drawImage(image, x, y);
            }
            this.context.restore();
        };
        Renderer.prototype.drawImageRotated = function (image, r, x, y, w, h, rx, ry) {
            if (rx === void 0) { rx = 0; }
            if (ry === void 0) { ry = 0; }
            this.context.save();
            this.context.translate(x + rx, y + ry);
            this.context.rotate(r);
            this.context.drawImage(image, -rx, -ry, w, h);
            this.context.restore();
        };
        Renderer.prototype.drawStarburstAt = function (x, y, r) {
            var img = RogueRpg.Images.get("Starburst");
            this.starburstRotation += (2 * Math.PI) * (RogueRpg.Game.deltaTime / 8000); // One rotation per 4 seconds
            this.drawImageRotated(img, this.starburstRotation, x - r, y - r, r * 2, r * 2, r, r);
        };
        Renderer.prototype.drawShardGainLoop = function () {
            // this.context.drawImage(this.game.screenCaptureAfterDeath, 0, 0, this.ww, this.wh);
            this.context.clearRect(0, 0, this.ww, this.wh);
            if (this.game.relicAnimationPhase == "FadingIn") {
                this.context.fillStyle = "rgba(0, 0, 0, " + this.game.relicStateTimer + ")";
                this.context.fillRect(0, 0, this.ww, this.wh);
            }
            else {
                this.context.fillStyle = "rgba(0, 0, 0, 0.6)";
                this.context.fillRect(0, 0, this.ww, this.wh);
            }
            this.drawStarburstAt((this.ww / 2), 173, 150);
            this.context.font = "24px arial";
            var shardText = RogueRpg.Helpers.formatNumber(this.game.storedShards);
            var texWid = this.context.measureText(shardText).width;
            var boxWid = Math.max(texWid + 40, 65);
            this.context.strokeStyle = "white";
            this.context.lineWidth = 3;
            this.context.fillStyle = "black";
            this.context.fillRect((this.ww - boxWid) / 2, 155, boxWid, 35);
            this.context.strokeRect((this.ww - boxWid) / 2, 155, boxWid, 35);
            this.context.drawImage(RogueRpg.Images.get("shards"), (this.ww - boxWid) / 2 + 5, 160, 25, 25);
            this.context.fillStyle = "white";
            this.context.fillText(shardText, (this.ww + boxWid) / 2 - texWid - 8, 160);
            this.renderShardsSection = true;
            this.drawShardsSection();
            this.context.fillStyle = "rgb(255, 255, 255)";
            this.context.font = "28px arial";
            this.context.fillText("+ " + this.game.bonusShards, 215, this.wh - 45);
        };
        Renderer.offsetMap = {
            "Goblin": 12,
            "Bat": 11,
            "Boarman": 6,
            "Mud Clown": 6
        };
        Renderer.HealingPotionSize = 55;
        Renderer.HealingPotionRightOffset = 155;
        Renderer.HealingPotionBottomOffset = 103;
        Renderer.GemstoneSectionStart = { left: 20, bottom: 40, height: 21, width: 65 };
        Renderer.ShardSectionStart = { left: 100, bottom: 40, height: 21, width: 80 };
        Renderer.PrismaSectionStart = { left: 20, bottom: 70, height: 21, width: 65 };
        Renderer.GoldSectionStart = { left: 100, bottom: 70, height: 21, width: 80 };
        Renderer.FloorSectionSpot = { left: 10, top: 20, height: 26, width: 67 };
        // Draws an overlay with the item count on the inventory button
        // 100px from bottom
        // 60px from right
        Renderer.ItemCountPosition = { bottom: 90, right: 50, width: 20, height: 20 };
        return Renderer;
    }());
    RogueRpg.Renderer = Renderer;
    var Animation = /** @class */ (function () {
        function Animation(name, drawX, drawY, maxTime, rotation) {
            if (rotation === void 0) { rotation = 0; }
            this.timeElapsed = 0;
            this.name = name;
            this.maxTime = maxTime;
            this.drawX = drawX;
            this.drawY = drawY;
            this.rotation = rotation;
        }
        return Animation;
    }());
    RogueRpg.Animation = Animation;
})(RogueRpg || (RogueRpg = {}));
//# sourceMappingURL=Render.js.map